from pylab import *
rc('font',   size=12)

def load_data(fname):
    return loadtxt(fname, dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })

def set_ticks(ax, t):
    ax.set_xticks(t)
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)

fig,gs = subplots(2, 2, figsize=(8,6))
handles = []

#--- Schur PC ---
data = loadtxt('dmem_data/mn4/schur.dat', dtype={
    'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
    'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
    })

for omp in sorted(unique(data['omp'])):
    if omp == 48: continue

    d = data[data['omp'] == omp]
    c = unique(d['mpi'] * omp)
    m = unique(d['mpi'])

    setup = array([min(d[d['mpi']==i]['setup']) for i in m])
    solve = array([min(d[d['mpi']==i]['solve']) for i in m])
    iters = array([min(d[d['mpi']==i]['iters']) for i in m])
    total = setup + solve

    ax = gs[0,0]
    h = ax.loglog(c, total, '.-')
    ax.set_ylim([1e0, 5e2])
    if omp==24:
        ideal = total[0] * c[0] / c
        hi = ax.plot(c, ideal, 'k:', zorder=1, linewidth=1, alpha=0.5)
    handles.append(h[0])

    ax = gs[0,1]
    ax.loglog(c, setup, '.-')
    if omp==24:
        ideal = setup[0] * c[0] / c
        ax.plot(c, ideal, 'k:', zorder=1, linewidth=1, alpha=0.5)
    ax.set_ylim([5e-2, 5e2])

    ax = gs[1,0]
    ax.loglog(c, solve, '.-')
    if omp==24:
        ideal = solve[0] * c[0] / c
        ax.plot(c, ideal, 'k:', zorder=1, linewidth=1, alpha=0.5)
    ax.set_ylim([1e0, 5e2])

    ax = gs[1,1]
    ax.semilogx(c, iters, '.-')
    ax.set_ylim([0,110])

#--- Trilinos ---
d = loadtxt('dmem_data/mn4/ns_trilinos.dat', dtype={
        'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
        'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
        })

m = d['mpi']

setup = d['setup']
solve = d['solve']
iters = d['iters']
total = setup + solve

h = gs[0,0].loglog(m, total, '.-')
handles.append(h[0])

gs[0,1].loglog(m, setup, '.-')
gs[1,0].loglog(m, solve, '.-')
gs[1,1].semilogx(m, iters, '.-')

for i in range(2):
    for j in range(2):
        set_ticks(gs[i,j], [96 * 2**k for k in range(7)])

gs[0,0].set_ylabel('Total time')
gs[0,1].set_ylabel('Setup time')

gs[1,0].set_ylabel('Solve time')
gs[1,0].set_xlabel('Number of cores (MPI * OMP)')

gs[1,1].set_ylabel('Iterations')
gs[1,1].set_xlabel('Number of cores (MPI * OMP)')

fig.tight_layout()

figlegend(handles + [hi[0]], ['OMP={}'.format(i) for i in (1, 4, 12, 24)] +
        ['Trilinos', 'Ideal scaling'],
       ncol=3, loc='lower center')
fig.suptitle('Strong scaling of the Navier-Stokes problem on MareNostrum 4 cluster')
fig.subplots_adjust(top=0.93, bottom=0.2)

show()